% DPROEQLB: One iteration of the backward Lyapunov equation that determines
%           the deterministic parameter optimal reduced order LQG compensator.
%       
%           [pp,ss,f,k,l,sigp,sigs,tps]=dproeqlb(pp,ss,p,g,c,q,r,v,w,mc,me)
%
%           Input:  p,g,c,q,r,v,w; LQG problem parameters.
%                   pp; Augmented system second moment matrix.
%                   ss; Lagrange multipliers.
%
%           Output: pp; Augmented system second moment matrix.
%                   ss; Lagrange multipliers.
%                   f,k,l: compensator matrices.
%                   sigp, sigs: compensator costs.
%                   tps       : trace(pt+st).
%
%           See also dprotvl, dproeqlf
%
%           L.G. van Willigenburg, W.L. de Koning 27-06-'96.
%
  function [pp,ss,f,k,l,sigp,sigs,tps]=dproeqlb(pp,ss,p,g,c,q,r,v,w,mc,me)

  [nxn,nx]=size(p); [nt,nt]=size(pp); nc=nt-nx; nx1=nx+1;
  p1=pp(1:nx,1:nx);
  if nt==nx;
    ph=zeros(nx); f=[]; l=[]; pf=1;
  else
    p12=pp(1:nx,nx1:nt); p21=pp(nx1:nt,1:nx);
    p2=pp(nx1:nt,nx1:nt); pip2=pinv(p2);
    ph=p12*pip2*p21; gg=pip2*p21; pf=0;
  end;
  pt=p1-ph;

  [nt,nt]=size(ss); ncn=nt-nxn; nxn1=nxn+1;
  s1=ss(1:nxn,1:nxn); 
  if nt==nxn
    sh=zeros(nxn); f=[]; k=[]; sf=1;
  else
    s12=ss(1:nxn,nxn1:nt); s21=ss(nxn1:nt,1:nxn);
    s2=ss(nxn1:nt,nxn1:nt); pis2=pinv(s2);
    sh=s12*pis2*s21; hh=-pis2*s21; sf=0;
  end
  st=s1-sh;
  
  if pf==1
    k0=(me+p*pt*c')*pinv(c*pt*c'+w); k=hh*k0;
    pa=[p; k*c]; qa=q;
    va=[v  me*k';  k *me' k *w*k'];
  elseif sf==1
    l0=pinv(g'*st*g+r)*(g'*st*p+mc'); l=l0*gg';
    pa=[p -g*l]; va=v;
    qa=[q -mc*l ; -l'*mc' l'*r*l];
  elseif min(size(r))==0
    k0=(me+p*pt*c')*pinv(c*pt*c'+w); k=hh*k0;
    f=hh*(p-k0*c)*gg'; l=[];
    pa=[p zeros(nxn,nc); k*c f];
    qa=[q zeros(nx,nc); zeros(nc,nx+nc)]; 
    va=[v  me*k';  k *me' k *w*k'];
  elseif min(size(w))==0
    l0=pinv(g'*st*g+r)*(g'*st*p+mc'); l=l0*gg';
    f=hh*(p-g*l0)*gg'; k=[];
    pa=[p -g*l; zeros(ncn,nx) f];
    qa=[q -mc*l ; -l'*mc' l'*r*l];
    va=[v zeros(nxn,ncn); zeros(ncn,nxn+ncn)];
  else
    l0=pinv(g'*st*g+r)*(g'*st*p+mc');
    k0=(me+p*pt*c')*pinv(c*pt*c'+w);
    f=hh*(p-k0*c-g*l0)*gg'; k=hh*k0; l=l0*gg';
    pa=[p -g*l; k*c f];
    qa=[q -mc*l ; -l'*mc' l'*r*l ];
    va=[v  me*k';  k *me' k *w*k'];
  end
  tps=trace(pt)+trace(st);
  sigp=trace(qa*pp); sigs=trace(va*ss);

  ss=pa'*ss*pa +qa; %pp=pa *pp*pa'+va;
