% DPROTVA: Deterministic Parameter Reduced Order Time Varying LQG compensation.
%          Optimal compensation of (P(i),G(i),C(i),V(i),W(i),x0m,x0c) based on
%         (Q(i),R(i),H) and nc<=nx for asynchronously sampled systems.
%         (P(i),G(i),C(i)) deterministic.
%
%         [xfkl,sigp,sigs,ptt,stt,pht,sht]=dprotva(lqgarr,nc,opt);
%         or
%         [xfkl,sigp,sigs,ptt,stt,pht,sht]=dprotva(lqgarr,nc,opt,ptt,stt,pht,sht);
%
% Input:
%          lqgarr: column vector containing the lqg problem data
%          nc    : row vector with dimensions of the compensator state
%          opt   : optional array containing algorithm parameter settings.
%          opt(1): convergence tolerance, epsl, default 1e-6. 
%          opt(2): iteration limit, maxtps, default 1e20.
%          opt(3): maximum number of iterations (sweeps), itm, default 5000. 
%          opt(4): plot parameter, itc, default 0.
%                  0    : no plot.
%                  <0   : plot at the end
%                  >0   : plot every itc iterations.
%          opt(5): numerical damping coefficient, 0<b<1, default 0.25.
%                  if opt(5)>1 | opt(5)<0 then b=0
%          opt(6): initial conditions ph, sh, ic=1,2,3, default 1
%                  ic=1: 0.9*eye(nx)+0.1*ones(nx)
%                  ic=2: random
%                  ic=3: eye(nx)
%          opt(7): method to compute projection, m=1,2,3, default 1
%                  m=1: Eigenvalues
%                  m=2: SVD's
%                  m=3: Group inverse
%          opt(8): relative tolerance rank(ph*sh), rtol default 1e-9
%          opt(9): not used
%         opt(10): stepsize homotopy parameter, default 1
%         opt(10): if 0 then the full-order LQG compensator is computed
%      opt(10:..): individual values homotopy parameter, default none
% ptt,stt,pht,sht: initial values ptt, stt, pht, sht (optional).
%
% Output:
%          xfkl : Optimal compensator in array format (see dprotv, getfkl)
%          sigp,sigs: Minimum costs (if sigp=sigs)
%          ptt, stt, pht, sht: Solution optimal projection equations.
%
% Comments:
%          See also dproeqc dproeqe dpcomco gmhfac dxfkl mintv ctrbtv obsvtv
%
%          L.G. Van Willigenburg, W.L. De Koning, 28-11-95.
%
  function [xfkl,sigp,sigs,ptt,stt,pht,sht,nsweeps]=dprotva(lqgarr,nc,opt,ptt,stt,pht,sht);
  
%
% Check number of input arguments and get
% problem data from lqgarr
%
  if nargin==2; no=0;
  elseif nargin==3; no=max(size(opt));
  elseif nargin==7; no=max(size(opt));
  else; error('two,three or seven input arguments required'); end;
  [nh,mh]=size(lqgarr); N=lqgarr(1);
  if mh~=1 ;
    error(' First input argument must be a column vector');
  end;
  [nh,mh]=size(nc);
  if nh~=1 | mh<N-1;
    error(' Second input argument must be a row vector with length >=N-1');
  end; nc(N)=0; nc=nc(1:N);
%
% Algorithm parameter settings.
%
  opth=zeros(1,8);
  if no~=0; opth(1:no)=opt(1:no); end;
  opt=opth;
  if opt(1)>0; epsl=opt(1); else; epsl=1e-6; end
  if opt(2)>0; maxtps=opt(2); else; maxtps=1e20; end
  if opt(3)>0; itm=opt(3); else; itm=5000; end
  if opt(4)~=0; itc=opt(4); else; itc=0; end
  if opt(5)==0; b=0.25;
  elseif opt(5)<0 | opt(5)>1; b=0;
  else b=opt(5); end; bb=b;
  if opt(6)>=1 & opt(6)<=3; ic=round(opt(6)); else; ic=1; end
  if opt(7)>=1 & opt(7)<=3; m=round(opt(7)); else; m=1; end
  if opt(8)>0; rtol=opt(8); else; rtol=1e-9; end;
  if no==10; if opt(10)<0 | opt(10)>1;
               error('0 <= opt(10) <= 1 not satisfied')
             elseif opt(10)~=0; da=1/round(1/opt(10)); av=[0:da:1];
             else; av=0;
             end;
  elseif no>10; av=sort(opt(10:no));
                av=av-av(1); av=av/av(no-9);
  else; av=1; end;
  
% Dimension check and index array computation
  nxt=0; inps=nxt; nxm=0;
  for i=1:N
    [nx]=getedo(lqgarr,i); nxm=max(nx,nxm);
    if av==0; nc(i)=nx; else; nc(i)=min(nc(i),nx); end;
    nxt=nxt+nx*nx; inps=[inps; nxt];
    if nc(i)<0
       error([' =nc(' num2str(i) ')<0']);
    end;
  end

% Set initial pt, st, ph ,sh if necessary
  if nargin~=6
   [ptt,stt,pht,sht]=inipstva(lqgarr,nc,ic);
  end;
  
% Boundary conditions recursions
  [x0m,x0c,H]=getxgh(lqgarr);
  
% Initialization other variables
  nsweeps=0; endloop=0; tr=1; trr=1; trt=[];
  tat=zeros(length(ptt),1); trps=[];

% St,Sh,Pt,Ph loop
  for a=av; endloop=0; conver=0;
  while endloop~=1

    nsweeps=nsweeps+1;

    tro=tr;
    for i=1:N
      pn1=inps(i)+1; pn2=inps(i+1);
      [nx]=getedo(lqgarr,i);
      ph=zeros(nx); sh=ph;
      ph(:)=pht(pn1:pn2,1); sh(:)=sht(pn1:pn2,1);
      [ta]=gmhfac(ph,sh,nc(i),a,m,rtol);
      tat(pn1:pn2,1)=ta(:);
    end;

    if a~=0 & bb~=0; stto=stt; shto=sht; end;
    pn1=inps(N)+1; pn2=inps(N+1);
    [nx]=getedo(lqgarr,N);
    st=zeros(nx); sh=st;
    st(:)=stt(pn1:pn2,1); sh(:)=sht(pn1:pn2,1);

    for i=N-1:-1:1
      [nx,nxn,mi,ldi,p,g,q,r,mc,v,c,w,me]=getedo(lqgarr,i);
      pn1=inps(i)+1; pn2=inps(i+1);
      pt=zeros(nx); ph=pt; ta=pt;
      pt(:)=ptt(pn1:pn2,1); ph(:)=pht(pn1:pn2,1); ta(:)=tat(pn1:pn2,1);
      [st,sh]=dproeqc(p,g,c,v,w,q,r,mc,me,st,sh,pt,ph,ta,1);
      stt(pn1:pn2,1)=st(:); sht(pn1:pn2,1)=sh(:);
    end
    if a~=0 & bb~=0; stt=(1-bb)*stt+bb*stto; sht=(1-bb)*sht+bb*shto; end;
    
    if a~=0 & bb~=0; ptto=ptt; phto=pht; end;
    pn1=inps(1)+1; pn2=inps(2);
    [nx]=getedo(lqgarr,1);
    pt=zeros(nx); ph=pt;
    pt(:)=ptt(pn1:pn2,1); ph(:)=pht(pn1:pn2,1);
    for i=1:N-1
      [nx,nxn,mi,ldi,p,g,q,r,mc,v,c,w,me]=getedo(lqgarr,i);
      pn1=inps(i+1)+1; pn2=inps(i+1+1);
      st=zeros(nxn); sh=st; ta=st;
      st(:)=stt(pn1:pn2,1); sh(:)=sht(pn1:pn2,1); ta(:)=tat(pn1:pn2,1);
      [pt,ph]=dproeqe(p,g,c,v,w,q,r,mc,me,st,sh,pt,ph,ta,1);
      ptt(pn1:pn2,1)=pt(:); pht(pn1:pn2,1)=ph(:);
    end
    if a~=0 & bb~=0; ptt=(1-bb)*ptt+bb*ptto; pht=(1-bb)*pht+bb*phto; end;
    
    pn1=inps(1)+1; pn2=inps(2);
    [nx]=getedo(lqgarr,1);
    st=zeros(nx); st(:)=stt(pn1:pn2,1);
    tr=trace(pt)+trace(st); trt=[trt tr];
    trr=abs((tr-tro)/(tr+tro));
%
% Test convergence.
%
    if nsweeps>itm
      disp('Maximum number of iterations (sweeps) exceeded');
      endloop=1; sigp=Inf; sigs=sigp; sigc=sigp;
    elseif trr>maxtps
      disp('Algorithm failed, divergence');
      endloop=1; sigp=Inf; sigs=sigp; sigc=sigp;
    elseif trr<epsl;
      conver=conver+1; sigp=0;
    end

    if conver==2; bb=0.9*b;
    elseif conver==3; endloop=1;
    else; bb=b; end;
%
% Plot progress computation.
%
    if (itc>0 & rem(nsweeps,itc)==0) | (itc~=0 & endloop==1)
      str=['DPROTVA alpha=' num2str(a)];
      plconv(trt,trr,epsl,endloop,str,'Trace(P(N)+S(0))',nxm,nc);
    end;
    [flag]=fileflag('stop.txt');
    if flag; error(' Termination due to stop.txt'); end
  end;
  trps=[trps tr];
  end;
  delete stop.txt
  
  if ~isinf(sigp)
%
% Final update tat, tgt, tht
%
    tgt=[]; tht=[]; nxt=0; ingh=0;
    for i=1:N
      pn1=inps(i)+1; pn2=inps(i+1);
      [nx]=getedo(lqgarr,i);
      ph=zeros(nx); sh=ph; ta=ph;
      ph(:)=pht(pn1:pn2,1); sh(:)=sht(pn1:pn2,1); ta(:)=tat(pn1:pn2,1);
      [ta,tg,th]=gmhfac(ph,sh,nc(i),a,m,rtol);
      tat=[tat; ta(:)]; tgt=[tgt; tg(:)]; tht=[tht; th(:)];
      nxt=nxt+nx*size(tg,1); ingh=[ingh; nxt];
    end;
%
% Compute initial state compensator
%
    ind=N+1; indt=ind; xfkl=[];
    pn1=ingh(1)+1; pn2=ingh(1+1);
    [nx]=getedo(lqgarr,1);
    th=zeros(nc(1),nx); th(:)=tht(pn1:pn2,1);
    xc=th*x0m;
%
% Compute cost and compensator matrices
%
    sigp=0; sigs=0; pn2=nx; pn1=pn2-nx+1;
    for i=1:N-1
      [nx,nxn,mi,ldi,p,g,q,r,mc,v,c,w,me]=getedo(lqgarr,i);
      pn1=inps(i)+1; pn2=inps(i+1);
      pt=zeros(nx); ph=pt;
      pt(:)=ptt(pn1:pn2,1); ph(:)=pht(pn1:pn2,1);
      
      pn1=inps(i+1)+1; pn2=inps(i+1+1);
      st=zeros(nxn); sh=st;
      st(:)=stt(pn1:pn2,1); sh(:)=sht(pn1:pn2,1);

      [sigpi,sigsi,ko,lo]=dpcomco(p,g,c,q,r,v,w,mc,me,st,sh,pt,ph);
      sigp=sigp+sigpi; sigs=sigs+sigsi;

      pn1=ingh(i)+1; pn2=ingh(1+i);
      tg=zeros(nc(i),nx); tg(:)=tgt(pn1:pn2,1);
      
      pn1=ingh(i+1)+1; pn2=ingh(1+i+1);
      th=zeros(nc(i+1),nxn); th(:)=tht(pn1:pn2,1);
      
      if min(size(c))~=0 & min(size(g))~=0
        f=th*(p-g*lo-ko*c)*tg'; k=th*ko; l=lo*tg';
      elseif min(size(c))==0
        f=th*(p-g*lo)*tg'; k=[]; l=lo*tg';
      else
        f=th*(p-ko*c)*tg'; k=th*ko; l=[];
      end

      if i~=1
        xfkl=[xfkl; nc(i); nc(i+1); mi; ldi; f(:); k(:); l(:)];
        ind=ind+4+nc(i)*nc(i+1)+nc(i+1)*ldi+nc(i)*mi;
      else
        xfkl=[xfkl; nc(i); nc(i+1); mi; ldi; xc(:); f(:); k(:); l(:)];
        ind=ind+4+nc(i)+nc(i)*nc(i+1)+nc(i+1)*ldi+nc(i)*mi;
      end
      indt=[indt; ind];
    end
    xfkl=[N;indt;xfkl;nc(N)];
%
% Add costs associated to boundary conditions
%
    pn1=inps(1)+1; pn2=inps(1+1);
    [nx]=getedo(lqgarr,1);
    st=zeros(nx); sh=st;
    st(:)=stt(pn1:pn2,1); sh(:)=sht(pn1:pn2,1);

    pn1=inps(N)+1; pn2=inps(1+N);
    [nx]=getedo(lqgarr,N);
    pt=zeros(nx); ph=pt;
    pt(:)=ptt(pn1:pn2,1); ph(:)=pht(pn1:pn2,1);
    sigp=sigp+trace(H*(pt+ph));
    sigs=sigs+trace(x0c*(st+sh)+x0m*x0m'*st);
  end
