  function [phi,gam,qi,ri,am,vi,li,ti,xi,gk]=edottv(a,b,q,r,v,ts,y)
%EDOTTV: Equivalent Discrete Optimal Control Problem Time varying case
%
%    [phi,gam,qi,ri,am,vi,li,ti,xi,gk]=edottv(a,b,q,r,v,ts,y)
%
%    function to calculate equivalent discrete optimal tracking matrices
%    for one sample interval [t ,t   ) for a linear time varying stochastic
%                              k  k+1
%    system A,B,W with quadratic cost J(Q,R) using the trapezium numerical
%    integration rule and a second order approximation of exp(A*dt)
%    where dt is the stepsize of the numerical integration.
%
%    Tracking problem:
%
%    x = A(t) x + B(t) u + W(t)
%    J = int(0,t )((x-y)'Q(t)(x-y)+u'R(t)u)+(x(t )-y'(t ))H(x(t )-y(t ))
%               f                               f      f       f     f
%
%    t , k=0,1,2,..,N-1: sampling instants, not necessarily equidistant
%     k
%
%    t = t
%     f   N
%
%    x(t ) = x :    Measured or reconstructed states
%       k     k
%
%    u(t)=u(t )=u , t <=t<t   , k=0,1,...,N-1:   Piecewise constant control
%            k   k   k     k+1
%
%    v = cov(W(t)):  Covariance matrix of white noise W
%    ni:  Number of numerical integration steps during [t ,t   )
%                                                        k  k+1
%    dt =(t   -t )/ni
%          k+1  k
%
%    Outputs:
%
%    phi,gam   : equivalent discrete system matrices
%    qi,ri,am,li,ti,xi  : equivalent discrete criterion matrices
%    vi        : equivalent discrete noise covariance matrix
%    gk        : average cost due to white noise
%
%    Equivalent discrete tracking problem:
%
%    x    = PHI x + GAM u
%     k+1      k k     k k
%
%    j = sum(k=0,N-1) (x'QI x  +  u' RI u + 2*x' AM u - 2 LI x -2 TI u + XI )
%                       k  k k     k   k k     k   k k      k k     k k    k
%
%                      + x'Hx -2y'(t )Hx  +y'(t )Hy(t )
%                         N  N      f   N      f     f
%
%    Inputs:
%
%    a         :[ A(t ) A(t +dt) A(t +2*dt) ... A(t   ) ]
%                    k     k        k              k+1
%    b         :[ B(t ) B(t +dt) B(t +2*dt) ... B(t   ) ]
%                    k     k        k              k+1
%    q         :[ Q(t ) Q(t +dt) Q(t +2*dt) ... Q(t   ) ]
%                    k     k        k              k+1
%    v         :[ V(t ) V(t +dt) V(t +2*dt) ... V(t   ) ]
%                    k     k        k              k+1
%    r         :[ R(t ) R(t +dt) R(t +2*dt) ... R(t   ) ]
%                    k     k        k              k+1
%    y         :[ y(t ) y(t +dt) y(t +2*dt) ... y(t   ) ]
%                    k     k        k              k+1
%
  if nargin~=7;error('edottv requires SEVEN input arguments');end
  [nt,mt]=size(a);n=sqrt(nt);ni=mt-1;
  if rem(n,1)~=0;error('edottv: size mismatch argument 1');end;
  [nh,mh]=size(b);m=nh/n;mm=m*m;
  if rem(m,1)~=0|mh~=mt;error('edottv: size mismatch argument 2');end;
  [nh,mh]=size(q);
  if nh~=nt|mh~=mt;error('edottv: size mismatch argument 3');end
  [nh,mh]=size(r);
  if nh~=mm|mh~=mt;error('edottv: size mismatch argument 4');end
  [nh,mh]=size(v);
  if nh~=nt|mh~=mt;error('edottv: size mismatch argument 5');end
  [nh,mh]=size(ts);
  if nh~=1|mh~=1;error('edottv: size mismatch argument 6');end
  [nh,mh]=size(y);
  if nh~=n|mh~=mt;error('edottv: size mismatch argument 7');end

  qi=zeros(n);vi=zeros(n);gam=zeros(n,m);am=zeros(n,m);amud=zeros(n,m);
  ri=zeros(m);li=zeros(1,n);ti=zeros(1,m);xi=0;trud=0.0;gk=0.0;
  phi=eye(n);e=eye(n);delt=ts/ni;delta=0.5*delt;
  
  [bh]=getmat(b,n,1);[qh]=getmat(q,n,1);[rh]=getmat(r,m,1);
  [vh]=getmat(v,n,1);[yh]=getmat(y,n,1);
  gamud=delta*bh;qiud=delta*qh;viud=delta*vh;
  riud=delta*rh;liud=delta*yh'*qh;tiud=zeros(1,m);
  xiud=delta*yh'*qh*yh;

  for i=1:ni 
    ind=i+1;
    [ah1]=getmat(a,n,i);[ah2]=getmat(a,n,ind);[bh]=getmat(b,n,ind);
    [qh]=getmat(q,n,ind);[vh]=getmat(v,n,ind);[rh]=getmat(r,m,ind);
    [yh]=getmat(y,n,ind);
    aud=delta*(ah1+ah2);
    phiud=e+aud+0.5*aud*aud;phi=phiud*phi;
    gam=phiud*(gam+gamud);     gamud=delta*bh;              gam=gam+gamud;
    vi=phiud*(vi+viud)*phiud'; viud=delta*vh;               vi=vi+viud;
    qi=qi+qiud;                qiud=delta*phi'*qh*phi;      qi=qi+qiud;
    gk=gk+trud;                trud=delta*trace(vi*qh);     gk=gk+trud;
    am=am+amud;                amud=delta*phi'*qh*gam;      am=am+amud;
    ri=ri+riud;                riud=delta*(rh+gam'*qh*gam); ri=ri+riud;
    li=li+liud;                liud=delta*yh'*qh*phi;       li=li+liud;
    ti=ti+tiud;                tiud=delta*yh'*qh*gam;       ti=ti+tiud;
    xi=xi+xiud;                xiud=delta*yh'*qh*yh;        xi=xi+xiud;
end
