function [fbt,uft,kt,phit,gamt,dit,n,l,m,cost,costlq]=lqgatv(func,conmat,obsmat,pars)
% LQGATV : Function that computes a digital optimal LQG
%          controller for an asynchronous aperiodically sampled linear
%          time-varying system and a cost-functional with references for
%          both the state and control. All matrices and references are
%          specified in the function func.
%
%          [fbt,uft,kt,phit,gamt,dit,n,l,m,cost,costlq]=..
%          lqgatv(func,conmat,obsmat,pars)
%
%          Input: 	
%          conmat=[t0  t1  t2 .. tf;
%                  n0  n1  n2    nf;
%                  s0  s1  s2 .. sf;
%                  i01 i11 i21   if1;
%                  i02 i12 i22   if2;
%                   .   .   .     .
%                  i0m i1m i2m ..ifm]
%                  t0,t1,..,tf: Controlinstances and final time
%                  n0,n1,..,nf: Number of integr. steps within control t(i),t(i+1)
%                  s0,s1,..,sf: Number of updated controlvariables at t0,t1,..,tf
%                  i01..i0m   : Indices of the original controlvariables the first
%                  ones being updated.
%
%         obsmat=[t0  t1  t2 .. tf;
%                 s0  s1  s2 .. sf;
%                 i01 i11 i21   if1;
%                 i02 i12 i22   if2;
%                  .   .   .     .
%                 i0m i1m i2m ..ifm]
%                 t0,t1,..,tf: Observation instances
%                 s0,s1,..,sf: Number of outputs sampled
%                 i01..i0m   : Indices of the original outputs
%                 the first ones being sampled.
%
%         func:   String containing the function name of the function
%                 that describes the dynamics and the costcriterion
%                 see e.g. lqgady.m.
%         pars:   Parameters which can be passed to functions (func)
%
%         x0  :   Initial state.
%         G   :   Covariance of x0.
%
%         Output:
%         fbt :   Feedback gain matrices [fb1 fb2 .. ].
%         uft :   Feedforward vectors [uf1 uf2 ..].
%         kt  :   Kalman gains of estimator [k1 k2 ..].
%         n   :   Dimension of (original) system.
%         m   :   Dimension of the (original) control.
%         l   :   Dimension of the (original) output.
%         cost:   Total costs LQG problem.
%         costlq :Deterministic / LQ part of the cost
%
%         The total LQ costs are:
%         x0'*si*x0-2*x0'*vk+wk : Deterministic part cost of the LQG problem.

%
%	Computation of recursive solution
%
if nargout~=10 & nargout~=11 ;error('lqgatv requires 10 or 11 output arguments');end;
if nargin<3 |nargin>4;error('lqgatv requires 3 or 4 input arguments');end;
if nargin==3;pars=0;end;
[nl,ml]=size(conmat);steps=ml-1;tf=conmat(1,ml);
[ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w,l,h,G,x0]=feval(func,tf,pars);
nmax=n+m-1;
vk=h*xrh;si=h;wk=xrh'*h*xrh;costo=0;gks=0;
fbt=[];uft=[];kkt=[];phit=[];gamt=[];dit=[];
%clc;
for i=steps:-1:1
  tis=conmat(1,i);tif=conmat(1,i+1);ni=conmat(2,i);
  ts=tif-tis;time=[tis:(tif-tis)/ni:tif];
%
%	Obtain continuous dynamics
%
  [ah,bh,vh,qh,rh,xrh,urh,dh]=feval(func,time,pars);
%
%	Convert to equivalent discrete-time problem and store
%	equivalent discrete-time system
%
  [phi,gam,qi,ri,am,vi,li,ti,xi,gk]=edottv(ah,bh,qh,rh,vh,ts,xrh);
  [n,m]=size(gam); dis=zeros(n,1);
%
%	Rearrange matrices to obtain the equivalent discrete-time problem
%   in the case of asynchronous and aperiodic sampling
%   
  [phi,gam,dis,qi,ri,am,vi,li,ti,xi,gk]=lqgare(i,conmat,phi,gam,dis,qi,ri,am,vi,li,ti,xi,gk);
  costo=costo+trace(vi*si)+gk; gks=gks+gk;
%
%	Computation of Ricatti type recursions
%
  sip=si;
  [al,uff,si,vk,wk]=srict(phi,gam,qi,ri,am,li,ti,si,vk,xi,wk);
  kk=al'*(ri+gam'*sip*gam)*al;
%
%	Store optimal feedback and feedforward
% 
  n1=n+m-conmat(3,i);
  m1=conmat(3,i);
  al=[al zeros(m1,nmax-n1);zeros(m-m1,nmax)];
  fbt=[al fbt];
  uff=[uff;zeros(m-m1,1)];
  uft=[uff uft];
  kkt=[kk(1:n,1:n) kkt];
end
costlq=x0'*si*x0-2*x0'*vk+wk;

% Initialize stochastic part of the cost
costo=costo+trace(si(1:n,1:n)*G);

%
%	Compute estimator and additional cost in stochastic case
%
[nio,maxio]=size(obsmat);
%
%	Initialization estimation, covariance and costs
%
tln=obsmat(1,1);tp=conmat(1,1);io=1;
Pcva=G;Ptot=G;kt=[];
kk=kkt(:,1:n);
costo=costo+trace(kk*Pcva);
%
%	If initial observation and control instant are equal
%	start with measurement update at initial instant.
%
if tp==tln
%
%	Obtain output and output covariance matrix
%
    [ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w]=feval(func,tp,pars);
%
%	Adjust output matrix and output covariance
%
    [c,w]=lqgaob(obsmat,io,c,w);
    [Pcva,kal,xhat]=kmfmu(c,w,Pcva);
    kt=[kt kal];
    tp=tln;io=io+1;
    if io<=maxio
      tln=obsmat(1,io);
    else
      tln=conmat(1,steps+1);
    end
end
%
%	Start estimator loop
%
for ic=2:steps
  tc=conmat(1,ic);
  dt=(conmat(1,ic)-conmat(1,ic-1))/conmat(2,ic-1);
%
%	while tc>=tln: time and measurement update at tln.
%
  while tc>=tln
    ni=round((tln-tp)/dt);time=[tp:(tln-tp)/ni:tln];
%
%	Obtain dynamics
%
    [ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w]=feval(func,time,pars);
%
%	Adjust output matrix and output covariance
%
    [c,w]=lqgaob(obsmat,io,c,w);
%
%	Compute equivalent discrete-time system
%
    [phi,gam,qi,ri,am,vi,li,ti,xi,gk]=edottv(ah,bh,qh,rh,vh,tln-tp,xrh);
    [n,m]=size(gam);phit=[phit phi];gamt=[gamt gam];%
%	Perform time and measurement update
%
    [Pcva,xhat]=kmftu(phi,gam,vi,Pcva);
    if tc==tln
       kk=kkt(:,1+(ic-1)*n:ic*n);
       costo=costo+trace(kk*Pcva);
    end;
    if tln~=conmat(1,steps)
      [Pcva,kal,xhat]=kmfmu(c,w,Pcva);kt=[kt kal];
      tp=tln;io=io+1;
      if io<=maxio
        tln=obsmat(1,io);
      else
        tln=conmat(1,steps+1);
      end
    else
       tp=tln;tln=conmat(1,steps+1);
    end
  end;
  if tc~=tp
%
%	Perform time update at tc.
%
    ni=round((tc-tp)/dt);time=[tp:(tc-tp)/ni:tc];
%
%	Obtain dynamics
%
    [ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w]=feval(func,time,pars);
%
%	Compute equivalent discrete-time system
%
    [phi,gam,qi,ri,am,vi,li,ti,xi,gk]=edottv(ah,bh,qh,rh,vh,tc-tp,xrh);
    [n,m]=size(gam); dis=zeros(n,1); phit=[phit phi];gamt=[gamt gam];
%
%	Perform time update
%
    [Pcva,xhat]=kmftu(phi,gam,vi,Pcva);
    kk=kkt(:,1+(ic-1)*n:ic*n);

    costo=costo+trace(kk*Pcva);
    tp=tc;
 end;
end
% Compute total cost: Deterministic / LQ cost + Stochastic cost
cost=costlq+costo;