% LQGRARV : Function that computes equivalent discrete optimal LQG
%           regulator problem for an asynchronous aperiodically sampled linear
%           time-varying system. All matrices are specified in the
%           function func.
%
%           [lqgarr,gk]=lqgrarv(func,Tsamp)
%
%	Input: 	
%	Tsamp= [t0; cn; ci; on; co;
%	        t1; cn; ci; on; co;
%	        ..
%	        tN; cn; ci; on; co;]
%
%         ti : sampling instant
%	        cn : number of updated controls
%	        ci : indices of updated and non updated controls, columnvector
%	        on : number of outputs
%	        co : indices of outputs, columnvector
%
%	func:    String containing the function name of the function
%	         that  describes the dynamics and the costcriterion
%	         see e.g. lqgady.m.
%
%	Output:
%
%	lqgarr:  Array containing all matrices
%	         describing the equivalent discrete-time
%	         LQG problem
%	array format:
%	         [N;ind(1);..;ind(N);
%	         n(1);n(2);m(1);l(1);x0m(:);x0c(:);
%	         phi(1)(:);gam(1)(:);q(1)(:);r(1)(:)mc(1)(:);v(1)(:);
%	         n(2);n(3);m(2);l(2);
%	         phi(1)(:);gam(1)(:);q(1)(:);r(1)(:)mc(1)(:);v(1)(:);
%	         ...
%	         ...
%	         n(N-1);n(N);m(N-1);l(N-1);
%	         phi(N-1)(:);gam(N-1)(:);...;v(N-1)(:);
%	         n(N);H(:)];
%
%	         ind(i), i=1,2,..,N pointers to starting positions in lqgarr :
%	         the data for time i starts at lqgarr(ind(i)+1) or at
%	         lqgarr(lqgarr(i+1)+1)
%    gks  :  Costs independent of optimization


function [lqgarr,gks]=lqgrarv(func,Tsamp)
dt=0.01; [n1,N]=size(Tsamp); tis=Tsamp(1,1); igco=2; gks=0;
[ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w,l,H,x0c,x0m]=feval(func,tis);
Asamp=sm2arr(Tsamp,m); % Convert sampling matrix to array
ni1=0; lqgarr=[]; ind=N+1; indt=ind;
for i=1:N-1
  [tis,nc,no,ic,io,igco]=getcob(Asamp,m,l,igco);
  tif=Asamp(igco);
  if i==N-1; nc1=m; ic1=1:m;
  else; nc1=Asamp(igco+1); ic1=Asamp(igco+2:igco+m+1); end;
  if nc1>m; error(' nc1 > m'); end;
  if nc1<0; error(' nc1 < 0'); end;
  ts=tif-tis; dti=ts/ceil(ts/dt); tim=[tis:dti:tif];
%
%	Obtain continuous dynamics
%
  [ah,bh,vh,qh,rh,xrh,urh,dh,n,m,c,w]=feval(func,tim,1);
%
%	Convert to conventional equivalent discrete-time problem
%
  [phi,gam,qi,ri,mc,vi,gk]=edortv(ah,bh,qh,rh,vh,ts); me=zeros(size(c'));
  [n,m]=size(gam); dis=zeros(n,1); li=dis';
  ti=zeros(1,m); xi=0; gks=gks+gk;
%
%	Rearrange matrices to obtain the equivalent discrete-time problem
%   in the case of asynchronous and aperiodic sampling
%   
  [phi,gam,dis,qi,ri,mc,vi,li,ti,xi,gk,c,w,me]=lqgarea(nc,ic,nc1,ic1,phi,gam,dis,qi,ri,mc,vi,li,ti,xi,gk,no,io,c,w,me);
%
%   Store the equivalent discrete-time problem data in an array
%
  [ni,mi,nh,ldi]=pgcvchk(phi,gam,qi,ri,mc,vi,c,w,me);
  if i~=1;
    if ni~=ni1; error('  ni sequence error'); end;
    ind=ind+4+ni*nh+nh*mi+ni*ni+mi*mi+ni*mi+nh*nh+ldi*ni+ldi*ldi+nh*ldi;
    ni1=nh; lqgarr=[lqgarr; ni; ni1; mi; ldi; phi(:); gam(:); qi(:); ri(:); mc(:); vi(:); c(:); w(:); me(:)];
 else
    [nn,mm]=size(x0m);
    if mm~=1; error(' x0m should be a column vector'); end
    if nn~=ni error(' Dimensions x0m incompatible with p(1)'); end
    [nn,mm]=size(x0c);
    if nn~=n | mm~=n; error(' Dimensions x0c incompatible a'); end
    x0c=[x0c zeros(n,ni-n); zeros(ni-n,ni)];
    ind=ind+4+ni+ni*ni+ni*nh+nh*mi+ni*ni+mi*mi+ni*mi+nh*nh+ldi*ni+ldi*ldi+nh*ldi;
    ni1=nh; lqgarr=[lqgarr; ni; ni1; mi; ldi; x0m(:); x0c(:); phi(:); gam(:); qi(:); ri(:); mc(:); vi(:); c(:); w(:); me(:)];
  end
  indt=[indt; ind];
end;
[nn,mm]=size(H);
if nn~=ni1 | mm~=ni1;
   error(' Dimensions H incompatible with p(N-1)');
end;
lqgarr=[N;indt;lqgarr;ni1;H(:)];
